#!/bin/bash

# 15-01-2025 - Single End
#  microRNA flavor

# 30_se_mir.sh

#### parameters here need to be adjusted for specific runs
srcf="$HOME/MAF/data"		# /absolute path to input fastq data folder (gunzip)
ext="fastq"			# file extension
tf="$HOME/MAF/results/se_mir"		# /absolute path to result folder
seqA="0"			# seqcodeA: 0: no dedup, 1: unique sequnce reads by nubeam-dedup
umiB="0"			# barcodeB: 0: no dedup, 1: R by umi, 2: samtools markdup, 3: umi_tools
rscr="$HOME/bin/qname_srv.uni.R"	# R script deduplicate, only necessary when umiB="1"

gdiSA="$HOME/MAF/index/star2711x111HUmt"
gtfSA="$HOME/MAF/reference/mt.Homo_sapiens.miRtRNA.gtf"
tfaSA="$HOME/MAF/reference/mt.trans.hu.miRtRNA.111.primary.fa"

gdiSB="$HOME/MAF/index/star2711x111HUpmm"
gtfSB="$HOME/MAF/reference/Homo_sapiens.GRCh38.111.mirbase.gtf"

gdiSC="$HOME/MAF/index/star2711x111HUmte"
gtfSC="$HOME/MAF/reference/mte.Homo_sapiens.GRCh38.111.gtf"
tfaSC="$HOME/MAF/reference/mte.trans.hu.GRCh38.111.primary.fa"

wayBT1="--end-to-end"		# --local --end-to-end
idxBT1="$HOME/MAF/index/bt2_mt/bt2_mt"

wayBT2="--local"		# --local --end-to-end
idxBT2="$HOME/MAF/index/bt2_111HU/bt2_111HU"
gtfBT2="$HOME/MAF/reference/mt.Homo_sapiens.miRtRNA.gtf"	# for multimap only

wayBT3="--end-to-end"		# --local --end-to-end
idxBT3="$HOME/MAF/index/bt2_pir/bt2_pir"

refBBM="$HOME/MAF/reference/genome3_2711/m2_miRtRNA.fasta"
idxBBM="$HOME/MAF/index/bb_mt/"

#### system parameter - do not change except you know what you are doing
pfin="00raw"		# start folder name

#### master folder & simple read statistics & create structure & symbolic links - se
icreate () {
	# 00 create target folder  -  creates parent path if necessary
	# first delete folder if it already exists
	if [ -d $tf ]; then
	  rm -R $tf;
	fi

	# create main folder & start sub folder
	mkdir -p $tf/$pfin

	# create file list and new structure
	fp="$srcf/*.$ext"
	yourfilenames=`ls $fp`

	fc=1
	for eachfile in $yourfilenames
	do
	  echo $eachfile
	  # save control list
	  readnumber=$(((`wc -l < $eachfile` +1) /4 ))
	  echo $fc";"$eachfile";read number;"$readnumber >> $tf/filelist.txt
	  
	  tmp="$tf/$pfin/sample$fc"
	  mkdir $tmp
	  # create sym link
	  part2=$(basename "$eachfile")
	  ln -s "$eachfile" "$tmp/$part2"
	  
	  ((fc++))
	done

	# go to start  -  run further steps from this directory on
	cd $tf
	# create cmds directory
	mkdir cmds
}

#### quality control by fastqc
qualcon () {
	pffpA="01qcAout"
	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pffpA/$sample; done
	ls -v $pffpA | xargs -I% echo 'fastqc -o '${pffpA}'/%/ --noextract -f fastq '${pfin}'/%/*.fastq' > cmds/qc1.sh
	chmod +x cmds/qc1.sh
	cmds/qc1.sh
	# adjust next STAR start folder
}

#### preprocess data by fastp w/ umi
preprocess () {
	pffpB="02ppAout"
	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pffpB/$sample; done
	# del adapter and move umi to qname and remove reads <16
	ls -v $pffpB | xargs -I% echo 'fastp -i '${pfin}'/%/*.fastq -o '${pffpB}'/%/%.tmp -U --umi_loc=read1 --umi_len=8 -l 16 -a TGGAATTCTCGGGTGCCAAGG -j '${pffpB}'/%/fp1.json -h '${pffpB}'/%/fp1.html' > cmds/pp1.sh
	# trim CA at front and remove again reads <16
	ls -v $pffpB | xargs -I% echo 'fastp -i '${pffpB}'/%/%.tmp -o '${pffpB}'/%/%.fastq -f 2 -l 16 -j '${pffpB}'/%/fp2.json -h '${pffpB}'/%/fp2.html' >> cmds/pp1.sh
	## paired end reads need some more work
	chmod +x cmds/pp1.sh
	cmds/pp1.sh
	
	# clean up
	ls -v $pffpB | xargs -I% echo 'rm -r '${pffpB}'/%/%.tmp' > cmds/pp2.sh
	chmod +x cmds/pp2.sh
	cmds/pp2.sh
}

#### dedup by sequences
# subfunction is run directly before the alignment step starts
# and is running on fastq files
seqcodeA () {
# '$pffpB'/%/%.fastq
	if [ $seqA = "1" ]
	then
		# rename
		ls -v $pffpB | xargs -I% echo 'mv '$pffpB'/%/%.fastq '$pffpB'/%/%O.fastq' > cmds/sub0.sh
		# remove duplicates based on nubeam-dedup (seq based, reference free)
		ls -v $pffpB | xargs -I% echo 'nubeam-dedup -i '$pffpB'/%/%O.fastq -o '$pffpB'/%/%.fastq' >> cmds/sub0.sh
		
		chmod +x cmds/sub0.sh
		cmds/sub0.sh
		
		# clean up
		ls -v $pffpB | xargs -I% echo 'rm '$pffpB'/%/%O.fastq' > cmds/cu0.sh
		chmod +x cmds/cu0.sh
		cmds/cu0.sh
		
		echo ""; echo "########  dedup seq  done"; echo ""
	else
		echo ""; echo "########  no dedup action"; echo ""
	fi
}

#### amplification umi barcode
# subfunction is run from inside  staroutA staroutB staroutC bt2outA bt2outB bt2outC bbmoutA
# and is running inside those folders on the bam files
barcodeB () {
	local fileIN=$1
	local sfolder=$2
	local fileINN=$3
	if [ $umiB = "0" ]
	then
		# do not remove duplicates
		ls -v $sfolder | xargs -I% echo 'mv '${sfolder}'/%/'${fileIN}' '${sfolder}'/%/dd.bam' > cmds/sub1.sh
		echo ""; echo "########  no umi"; echo ""
	fi
	if [ $umiB = "1" ]
	then
		# remove duplicates based only on barcode (ek R)
		ls -v $sfolder | xargs -I% echo "samtools view "${sfolder}"/%/"${fileINN}" | awk -F'[ \t]+' '{ print \$1 }' > out.txt" > cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'Rscript '$rscr' out.txt outuni.txt' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'samtools view -b -N outuni.txt '${sfolder}'/%/'${fileINN}' > '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'samtools sort '${sfolder}'/%/dd.bam -o '${sfolder}'/%/dd1.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'rm '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'mv '${sfolder}'/%/dd1.bam '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		echo ""; echo "########  umi only"; echo ""
	fi
	if [ $umiB = "2" ]
	then
		# remove duplicates based on samtools markdup (mapping plus barcode)
		ls -v $sfolder | xargs -I% echo 'samtools markdup -r -s -f '${sfolder}'/%/dup.stat.txt -u --barcode-name '${sfolder}'/%/'${fileIN}' '${sfolder}'/%/dd.bam' > cmds/sub1.sh
		echo ""; echo "########  umi samtools markdup"; echo ""
	fi
	if [ $umiB = "3" ]
	then
		# umi_tools (served by pipx),  ? --read-length, maybe away
		ls -v $sfolder | xargs -I% echo 'umi_tools dedup --read-length --umi-separator=: -I '${sfolder}'/%/'${fileIN}' --output-stats='${sfolder}'/%/dedup_stat.txt -S '${sfolder}'/%/dd.bam -L '${sfolder}'/%/dedup.log' > cmds/sub1.sh
		echo ""; echo "########  umi_tools"; echo ""
	fi
	
	ls -v $sfolder | xargs -I% echo 'samtools index '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
	chmod +x cmds/sub1.sh
	cmds/sub1.sh
}

#### STAR - mt
staroutA () {
	pfstarA="03star1out"
	gdi=$gdiSA
	gtf=$gtfSA
	tfa=$tfaSA

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfstarA/$sample; done

	## single end reads  [adjust for dir]
	# transcriptomic coordinates bam (genomic coordinates sam)
	ls -v $pffpB | xargs -I% echo 'STAR --runThreadN 8 --genomeDir '$gdi' --sjdbGTFfile '$gtf' --readFilesIn '$pffpB'/%/%.fastq --outFilterMismatchNmax 0 --outFilterMultimapNmax 2 --outSAMmultNmax 1 --outFileNamePrefix '$pfstarA'/%/ --quantMode TranscriptomeSAM' > cmds/mapstA.sh
	chmod +x cmds/mapstA.sh
	cat cmds/mapstA.sh | parallel -P 3 	# 3*8=24 threads = server

	# SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfstarA | xargs -I% echo 'samtools sort '${pfstarA}'/%/Aligned.toTranscriptome.out.bam -o '${pfstarA}'/%/Alig.sort.bam' > cmds/sc3A.sh
	ls -v $pfstarA | xargs -I% echo 'samtools index '${pfstarA}'/%/Alig.sort.bam -o '${pfstarA}'/%/Alig.sort.bam.bai' >> cmds/sc3A.sh
	chmod +x cmds/sc3A.sh
	cmds/sc3A.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfstarA "Aligned.toTranscriptome.out.bam"
	
	# count
	ls -v $pfstarA | xargs -I% echo 'samtools index '${pfstarA}'/%/dd.bam -o '${pfstarA}'/%/dd.bam.bai' > cmds/sc3A.sh
	ls -v $pfstarA | xargs -I% echo 'samtools idxstats '${pfstarA}'/%/dd.bam > '${pfstarA}'/%/mcounts.tsv' >> cmds/sc3A.sh
	chmod +x cmds/sc3A.sh
	cmds/sc3A.sh

	# SALMON   tfa depends on STAR
	# SE single end
	ls -v $pfstarA | xargs -I% echo 'salmon quant -p 8 -t '$tfa' -l U -a '${pfstarA}'/%/dd.bam -o '${pfstarA}'/%' > cmds/sal1.sh
	chmod +x cmds/sal1.sh
	cmds/sal1.sh
	
	# clean up
	ls -v $pfstarA | xargs -I% echo 'rm -r '${pfstarA}'/%/aux_info' > cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm -r '${pfstarA}'/%/libParams' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm -r '${pfstarA}'/%/logs' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm -r '${pfstarA}'/%/_STARgenome' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/Alig.sort.bam.bai' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/Aligned.toTranscriptome.out.bam' >> cmds/cu1.sh		# comment it out if you want to keep it
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/Aligned.out.sam' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/dd.bam.bai' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/dd.bam' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/cmd_info.json' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/Log.out' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/Log.progress.out' >> cmds/cu1.sh
	ls -v $pfstarA | xargs -I% echo 'rm '${pfstarA}'/%/SJ.out.tab' >> cmds/cu1.sh
	chmod +x cmds/cu1.sh
	cmds/cu1.sh
}

#### STAR premature
staroutB () {
	pfstarB="03star2out"
	gdi=$gdiSB
	gtf=$gtfSB

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfstarB/$sample; done

	## single end reads  [adjust for dir]
	# transcriptomic coordinates bam (genomic coordinates sam)
	ls -v $pffpB | xargs -I% echo 'STAR --runThreadN 8 --genomeDir '$gdi' --sjdbGTFfile '$gtf' --readFilesIn '$pffpB'/%/%.fastq --outFilterMismatchNmax 0 --outFilterMultimapNmax 2 --outSAMmultNmax 1 --outFileNamePrefix '$pfstarB'/%/ --quantMode TranscriptomeSAM' > cmds/mapstB.sh
	chmod +x cmds/mapstB.sh
	cat cmds/mapstB.sh | parallel -P 3 	# 3*8=24 threads = server
	
	# pmm SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfstarB | xargs -I% echo 'samtools sort '${pfstarB}'/%/Aligned.toTranscriptome.out.bam -o '${pfstarB}'/%/Alig.sort.bam' > cmds/sc3B.sh
	ls -v $pfstarB | xargs -I% echo 'samtools index '${pfstarB}'/%/Alig.sort.bam -o '${pfstarB}'/%/Alig.sort.bam.bai' >> cmds/sc3B.sh
	chmod +x cmds/sc3B.sh
	cmds/sc3B.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfstarB "Aligned.toTranscriptome.out.bam"
	
	# count
	ls -v $pfstarB | xargs -I% echo 'samtools index '${pfstarB}'/%/dd.bam -o '${pfstarB}'/%/dd.bam.bai' > cmds/sc3B.sh
	ls -v $pfstarB | xargs -I% echo 'samtools idxstats '${pfstarB}'/%/dd.bam > '${pfstarB}'/%/mcounts.tsv' >> cmds/sc3B.sh
	chmod +x cmds/sc3B.sh
	cmds/sc3B.sh
	
	# clean up
	ls -v $pfstarB | xargs -I% echo 'rm -r '${pfstarB}'/%/_STARgenome' > cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/Alig.sort.bam.bai' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/Aligned.toTranscriptome.out.bam' >> cmds/cu2.sh		# comment it out if you want to keep it
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/Aligned.out.sam' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/dd.bam.bai' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/dd.bam' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/Log.out' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/Log.progress.out' >> cmds/cu2.sh
	ls -v $pfstarB | xargs -I% echo 'rm '${pfstarB}'/%/SJ.out.tab' >> cmds/cu2.sh
	chmod +x cmds/cu2.sh
	cmds/cu2.sh
}

#### STAR - mte
staroutC () {
	pfstarC="03star3out"
	gdi=$gdiSC
	gtf=$gtfSC
	tfa=$tfaSC
	
	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfstarC/$sample; done
	
	## single end reads  [adjust for dir]
	# transcriptomic coordinates bam (genomic coordinates sam)
	ls -v $pffpB | xargs -I% echo 'STAR --runThreadN 8 --genomeDir '$gdi' --sjdbGTFfile '$gtf' --readFilesIn '$pffpB'/%/%.fastq --outFilterMismatchNmax 0 --outFilterMultimapNmax 2 --outSAMmultNmax 1 --outFileNamePrefix '$pfstarC'/%/ --quantMode TranscriptomeSAM' > cmds/mapstC.sh
	chmod +x cmds/mapstC.sh
	cat cmds/mapstC.sh | parallel -P 3 	# 3*8=24 threads = server
	
	# SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfstarC | xargs -I% echo 'samtools sort '${pfstarC}'/%/Aligned.toTranscriptome.out.bam -o '${pfstarC}'/%/Alig.sort.bam' > cmds/sc3C.sh
	ls -v $pfstarC | xargs -I% echo 'samtools index '${pfstarC}'/%/Alig.sort.bam -o '${pfstarC}'/%/Alig.sort.bam.bai' >> cmds/sc3C.sh
	chmod +x cmds/sc3C.sh
	cmds/sc3C.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfstarC "Aligned.toTranscriptome.out.bam"
	
	# count
	ls -v $pfstarC | xargs -I% echo 'samtools index '${pfstarC}'/%/dd.bam -o '${pfstarC}'/%/dd.bam.bai' > cmds/sc3C.sh
	ls -v $pfstarC | xargs -I% echo 'samtools idxstats '${pfstarC}'/%/dd.bam > '${pfstarC}'/%/mcounts.tsv' >> cmds/sc3C.sh
	chmod +x cmds/sc3C.sh
	cmds/sc3C.sh

	# SALMON   tfa depends on STAR
	# SE single end
	ls -v $pfstarC | xargs -I% echo 'salmon quant -p 8 -t '$tfa' -l U -a '${pfstarC}'/%/dd.bam -o '${pfstarC}'/%' > cmds/sal3.sh
	chmod +x cmds/sal3.sh
	cmds/sal3.sh

	# clean up
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/aux_info' > cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/libParams' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/logs' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/_STARgenome' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Alig.sort.bam.bai' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Aligned.toTranscriptome.out.bam' >> cmds/cu3.sh		# comment it out if you want to keep it
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Aligned.out.sam' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/dd.bam.bai' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/dd.bam' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/cmd_info.json' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Log.out' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Log.progress.out' >> cmds/cu3.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/SJ.out.tab' >> cmds/cu3.sh
	chmod +x cmds/cu3.sh
	cmds/cu3.sh
}

#### Bowtie2 - mt
bt2outA () {
	pfbt2="04bt21out"
	way=$wayBT1
	idx=$idxBT1

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfbt2/$sample; done

	## single end - defaults --sensitive-local or --sensitive
	ls -v $pffpB | xargs -I% echo 'bowtie2 '$way' --threads 8 -x '$idx' -U '$pffpB'/%/%.fastq -S '$pfbt2'/%/a.sam &> '$pfbt2'/%/a.log' > cmds/mapbt2A.sh
	chmod +x cmds/mapbt2A.sh
	cat cmds/mapbt2A.sh | parallel -P 3 	# 3*8=24 threads = server
	
	# SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfbt2 | xargs -I% echo 'samtools view -bo '${pfbt2}'/%/Alig.bam '${pfbt2}'/%/a.sam' > cmds/sc4.sh
	ls -v $pfbt2 | xargs -I% echo 'samtools sort '${pfbt2}'/%/Alig.bam -o '${pfbt2}'/%/Alig.sort.bam' >> cmds/sc4.sh
	ls -v $pfbt2 | xargs -I% echo 'samtools index '${pfbt2}'/%/Alig.sort.bam -o '${pfbt2}'/%/Alig.sort.bam.bai' >> cmds/sc4.sh
	chmod +x cmds/sc4.sh
	cmds/sc4.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfbt2 "Alig.bam"
	
	# count
	ls -v $pfbt2 | xargs -I% echo 'samtools index '${pfbt2}'/%/dd.bam -o '${pfbt2}'/%/dd.bam.bai' > cmds/sc4.sh
	ls -v $pfbt2 | xargs -I% echo 'samtools idxstats '${pfbt2}'/%/dd.bam > '${pfbt2}'/%/mcounts.tsv' >> cmds/sc4.sh
	chmod +x cmds/sc4.sh
	cmds/sc4.sh
	
	# clean up
	ls -v $pfbt2 | xargs -I% echo 'rm '${pfbt2}'/%/a.sam' > cmds/cu3.sh
	ls -v $pfbt2 | xargs -I% echo 'rm '${pfbt2}'/%/Alig.bam' >> cmds/cu3.sh
	ls -v $pfbt2 | xargs -I% echo 'rm '${pfbt2}'/%/Alig.sort.bam.bai' >> cmds/cu3.sh
	ls -v $pfbt2 | xargs -I% echo 'rm '${pfbt2}'/%/dd.bam.bai' >> cmds/cu3.sh
	ls -v $pfbt2 | xargs -I% echo 'rm '${pfbt2}'/%/dd.bam' >> cmds/cu3.sh
	chmod +x cmds/cu3.sh
	cmds/cu3.sh
}

#### Bowtie2 - 111HU chr view mcounts - and qualimap mt  ..111HUmt.tsv counts
bt2outB () {
	pfbt2B="04bt22out"
	way=$wayBT2
	idx=$idxBT2
	gtfmt=$gtfBT2

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfbt2B/$sample; done

	## single end - defaults --sensitive-local or --sensitive
	ls -v $pffpB | xargs -I% echo 'bowtie2 '$way' --threads 8 -x '$idx' -U '$pffpB'/%/%.fastq -S '$pfbt2B'/%/a.sam &> '$pfbt2B'/%/a.log' > cmds/mapbt2B.sh
	chmod +x cmds/mapbt2B.sh
	cat cmds/mapbt2B.sh | parallel -P 3 	# 3*8=24 threads = server
	
	# SAM to BAM when @SQ lines are present in the header
	ls -v $pfbt2B | xargs -I% echo 'samtools view -bo '${pfbt2B}'/%/Alig.bam '${pfbt2B}'/%/a.sam' > cmds/mapbt2B2.sh
	ls -v $pfbt2B | xargs -I% echo 'samtools sort '${pfbt2B}'/%/Alig.bam -o '${pfbt2B}'/%/Alig.sort.bam' >> cmds/mapbt2B2.sh
	ls -v $pfbt2B | xargs -I% echo 'samtools index '${pfbt2B}'/%/Alig.sort.bam -o '${pfbt2B}'/%/Alig.sort.bam.bai' >> cmds/mapbt2B2.sh
	chmod +x cmds/mapbt2B2.sh
	cmds/mapbt2B2.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfbt2B "Alig.bam"
	
	# count
	ls -v $pfbt2B | xargs -I% echo 'samtools index '${pfbt2B}'/%/dd.bam -o '${pfbt2B}'/%/dd.bam.bai' > cmds/mapbt2B2.sh
	ls -v $pfbt2B | xargs -I% echo 'samtools idxstats '${pfbt2B}'/%/dd.bam > '${pfbt2B}'/%/mcounts.tsv' >> cmds/mapbt2B2.sh
	chmod +x cmds/mapbt2B2.sh
	cmds/mapbt2B2.sh
	
	# qualimap,   bam coordinate sorted (but not indexed)
	ls -v $pfbt2B | xargs -I% echo 'qualimap comp-counts -bam '${pfbt2B}'/%/dd.bam -gtf '$gtfmt' -out '${pfbt2B}'/%/qm.111HUmt.tsv' > cmds/mapbt2B3.sh
	chmod +x cmds/mapbt2B3.sh
	cmds/mapbt2B3.sh
	
	# clean up
	ls -v $pfbt2B | xargs -I% echo 'rm '${pfbt2B}'/%/a.sam' > cmds/cu4.sh
	ls -v $pfbt2B | xargs -I% echo 'rm '${pfbt2B}'/%/Alig.bam' >> cmds/cu4.sh
	ls -v $pfbt2B | xargs -I% echo 'rm '${pfbt2B}'/%/Alig.sort.bam.bai' >> cmds/cu4.sh
	ls -v $pfbt2B | xargs -I% echo 'rm '${pfbt2B}'/%/dd.bam.bai' >> cmds/cu4.sh
	ls -v $pfbt2B | xargs -I% echo 'rm '${pfbt2B}'/%/dd.bam' >> cmds/cu4.sh
	chmod +x cmds/cu4.sh
	cmds/cu4.sh
}

#### Bowtie2 - piRNA mcounts
bt2outC () {
	pfbt2C="04bt23out"
	way=$wayBT3
	idx=$idxBT3

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfbt2C/$sample; done

	## single end - defaults --sensitive-local or --sensitive
	ls -v $pffpB | xargs -I% echo 'bowtie2 '$way' --threads 8 -x '$idx' -U '$pffpB'/%/%.fastq -S '$pfbt2C'/%/a.sam &> '$pfbt2C'/%/a.log' > cmds/mapbt2C.sh
	chmod +x cmds/mapbt2C.sh
	cat cmds/mapbt2C.sh | parallel -P 3 	# 3*8=24 threads = server
	
	# SAM to BAM when @SQ lines are present in the header
	ls -v $pfbt2C | xargs -I% echo 'samtools view -bo '${pfbt2C}'/%/Alig.bam '${pfbt2C}'/%/a.sam' > cmds/mapbt2C2.sh
	ls -v $pfbt2C | xargs -I% echo 'samtools sort '${pfbt2C}'/%/Alig.bam -o '${pfbt2C}'/%/Alig.sort.bam' >> cmds/mapbt2C2.sh
	ls -v $pfbt2C | xargs -I% echo 'samtools index '${pfbt2C}'/%/Alig.sort.bam -o '${pfbt2C}'/%/Alig.sort.bam.bai' >> cmds/mapbt2C2.sh
	chmod +x cmds/mapbt2C2.sh
	cmds/mapbt2C2.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfbt2C "Alig.bam"
	
	# count
	ls -v $pfbt2C | xargs -I% echo 'samtools index '${pfbt2C}'/%/dd.bam -o '${pfbt2C}'/%/dd.bam.bai' > cmds/mapbt2C2.sh
	ls -v $pfbt2C | xargs -I% echo 'samtools idxstats '${pfbt2C}'/%/dd.bam > '${pfbt2C}'/%/mcounts.tsv' >> cmds/mapbt2C2.sh
	chmod +x cmds/mapbt2C2.sh
	cmds/mapbt2C2.sh
	
	# clean up
	ls -v $pfbt2C | xargs -I% echo 'rm '${pfbt2C}'/%/a.sam' > cmds/cu5.sh
	ls -v $pfbt2C | xargs -I% echo 'rm '${pfbt2C}'/%/Alig.bam' >> cmds/cu5.sh
	ls -v $pfbt2C | xargs -I% echo 'rm '${pfbt2C}'/%/Alig.sort.bam.bai' >> cmds/cu5.sh
	ls -v $pfbt2C | xargs -I% echo 'rm '${pfbt2C}'/%/dd.bam.bai' >> cmds/cu5.sh
	ls -v $pfbt2C | xargs -I% echo 'rm '${pfbt2C}'/%/dd.bam' >> cmds/cu5.sh
	chmod +x cmds/cu5.sh
	cmds/cu5.sh
}

#### BBMap - mt mcounts		[somewhat a global aligner]
bbmoutA () {
	pfbbmA="05bbm1out"
	ref=$refBBM
	path=$idxBBM

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfbbmA/$sample; done

	## single end - defaults --sensitive-local or --sensitive
	ls -v $pffpB | xargs -I% echo 'bbmap.sh in='$pffpB'/%/%.fastq ref='$ref' path='$path' out='$pfbbmA'/%/a.sam statsfile='$pfbbmA'/%/a.log covstats='$pfbbmA'/%/cs.txt' > cmds/mapbbm.sh
	chmod +x cmds/mapbbm.sh
	cmds/mapbbm.sh
	
	# SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfbbmA | xargs -I% echo 'samtools view -bo '${pfbbmA}'/%/Alig.bam '${pfbbmA}'/%/a.sam' > cmds/sc5.sh
	ls -v $pfbbmA | xargs -I% echo 'samtools sort '${pfbbmA}'/%/Alig.bam -o '${pfbbmA}'/%/Alig.sort.bam' >> cmds/sc5.sh
	ls -v $pfbbmA | xargs -I% echo 'samtools index '${pfbbmA}'/%/Alig.sort.bam -o '${pfbbmA}'/%/Alig.sort.bam.bai' >> cmds/sc5.sh
	chmod +x cmds/sc5.sh
	cmds/sc5.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfbbmA "Alig.bam"
	
	# count
	ls -v $pfbbmA | xargs -I% echo 'samtools index '${pfbbmA}'/%/dd.bam -o '${pfbbmA}'/%/dd.bam.bai' > cmds/sc5.sh
	ls -v $pfbbmA | xargs -I% echo 'samtools idxstats '${pfbbmA}'/%/dd.bam > '${pfbbmA}'/%/mcounts.tsv' >> cmds/sc5.sh
	chmod +x cmds/sc5.sh
	cmds/sc5.sh
	
	# clean up
	ls -v $pfbbmA | xargs -I% echo 'rm '${pfbbmA}'/%/a.sam' > cmds/cu6.sh
	ls -v $pfbbmA | xargs -I% echo 'rm '${pfbbmA}'/%/Alig.bam' >> cmds/cu6.sh
	ls -v $pfbbmA | xargs -I% echo 'rm '${pfbbmA}'/%/Alig.sort.bam.bai' >> cmds/cu6.sh
	ls -v $pfbbmA | xargs -I% echo 'rm '${pfbbmA}'/%/dd.bam.bai' >> cmds/cu6.sh
	ls -v $pfbbmA | xargs -I% echo 'rm '${pfbbmA}'/%/dd.bam' >> cmds/cu6.sh
	# remove adjusted  fastq  from preprocess
	ls -v $pffpB | xargs -I% echo 'rm '${pffpB}'/%/%.fastq' >> cmds/cu6.sh
	chmod +x cmds/cu6.sh
	cmds/cu6.sh
}

#### file name list -- (!) ev. needs adjustment
#    works on filelist.txt and creates namelist.txt for R import
fnlA () {
	fl="$(head -n 1 "$tf/filelist.txt")"
	tl=$(tr -dc '/' <<<"$fl" | wc -c)
	numE="$((tl+1))"
	fn=$(cut -d';' -f2 "$tf/filelist.txt" | cut -d'/' -f$numE | cut -d'.' -f1)
	echo $fn
	# save name list
	echo $fn > $tf/namelist.txt
}

#### main ####
dt1=$(date)

icreate
echo ""; echo "########  initialization -- done"; echo ""
qualcon
echo ""; echo "########  fastqc -- done"; echo ""
preprocess
echo ""; echo "########  fastp -- done"; echo ""
seqcodeA
# echo inside seqcodeA
staroutA
echo ""; echo "########  STAR -- mt + salmon + mcounts  done"; echo ""
staroutB
echo ""; echo "########  STAR -- premature + mcounts  done"; echo ""
staroutC
echo ""; echo "########  STAR -- mte + mcounts  done"; echo ""
bt2outA
echo ""; echo "########  BT2 -- mt + mcounts  done"; echo ""
bt2outB
echo ""; echo "########  BT2 -- 111HU + mcounts -- downstream mt qualimap mcounts  done"; echo ""
bt2outC
echo ""; echo "########  BT2 -- piRNA + mcounts  done"; echo ""
bbmoutA
echo ""; echo "########  BBM -- mt + mcounts  done"; echo ""
fnlA
echo ""; echo "########  namelist -- done"; echo ""

echo ""
dt2=$(date)
echo $dt1
echo $dt2


