#!/bin/bash

# 15-01-2025 - Paired End
#  mRNA flavor

# 30_pe_mrna.sh

#### parameters here need to be adjusted for specific runs
sue="_R1"		# adjust for pe: something like _F3 _F5 or _R1 _R2 in the fastq name
sue2="_R2"
srcf="$HOME/MAF/data"		# /absolute path to input fastq data folder (gunzip)
ext="fastq"			# file extension
tf="$HOME/MAF/results/pe_mrna"		# /absolute path to result folder
umiB="0"			# barcodeB: 0: no dedup, 1: R by umi, 2: samtools markdup, 3: umi_tools

gdiSC="$HOME/MAF/index/star2711x111HUmte"		# STAR index folder
gtfSC="$HOME/MAF/reference/mte.Homo_sapiens.GRCh38.111.gtf"		# gtf - feature information
tfaSC="$HOME/MAF/reference/mte.trans.hu.GRCh38.111.primary.fa"	# transcriptome fasta file

#### system parameter - do not change except you know what you are doing
pfin="00raw"		# start folder name

#### master folder & simple read statistics & create structure & symbolic links - pe
icreatePE () {
	# 00 create target folder  -  creates parent path if necessary
	# first delete folder if it already exists
	if [ -d $tf ]; then
	  rm -R $tf;
	fi

	# create main folder & start sub folder
	mkdir -p $tf/$pfin

	# create file list and new structure
	fp="$srcf/*$sue.$ext"
	yourfilenames=`ls $fp`
	fp2="$srcf/*$sue2.$ext"
	yourfilenames2=`ls $fp2`

	fc=1
	for eachfile in $yourfilenames
	do
	  echo $eachfile
	  # save control list
	  readnumber=$(((`wc -l < $eachfile` +1) /4 ))
	  echo $fc";"$eachfile";read number;"$readnumber >> $tf/filelist.txt
	  
	  tmp="$tf/$pfin/sample$fc"
	  mkdir $tmp
	  # create sym link
	  part2=$(basename "$eachfile")
	  ln -s "$eachfile" "$tmp/$part2"
	  
	  ((fc++))
	done
	
	fc=1
	for eachfile in $yourfilenames2
	do
	  echo $eachfile
	  # save control list
	  readnumber=$(((`wc -l < $eachfile` +1) /4 ))
	  echo $fc";"$eachfile";read number;"$readnumber >> $tf/filelist2.txt
	  
	  tmp="$tf/$pfin/sample$fc"
	  # create sym link
	  part2=$(basename "$eachfile")
	  ln -s "$eachfile" "$tmp/$part2"
	  
	  ((fc++))
	done

	# go to start  -  run further steps from this directory on
	cd $tf
	# create cmds directory
	mkdir cmds
}

#### quality control by fastqc - pe
qualconPE () {
	pffpA="01qcAout"
	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pffpA/$sample; done
	ls -v $pffpA | xargs -I% echo 'fastqc -o '${pffpA}'/%/ --noextract -f fastq '${pfin}'/%/*'$sue'.fastq' > cmds/qc1.sh
	ls -v $pffpA | xargs -I% echo 'fastqc -o '${pffpA}'/%/ --noextract -f fastq '${pfin}'/%/*'$sue2'.fastq' >> cmds/qc1.sh
	chmod +x cmds/qc1.sh
	cmds/qc1.sh
	# adjust next STAR start folder
}

#### amplification umi barcode
# subfunction is run from inside  staroutC
# and is running inside this folder on the bam file
barcodeB () {
	local fileIN=$1
	local sfolder=$2
	local fileINN=$3
	if [ $umiB = "0" ]
	then
		# do not remove duplicates
		ls -v $sfolder | xargs -I% echo 'mv '${sfolder}'/%/'${fileIN}' '${sfolder}'/%/dd.bam' > cmds/sub1.sh
		echo ""; echo "########  no umi"; echo ""
	fi
	if [ $umiB = "1" ]
	then
		# remove duplicates based only on barcode (ek R)
		ls -v $sfolder | xargs -I% echo "samtools view "${sfolder}"/%/"${fileINN}" | awk -F'[ \t]+' '{ print \$1 }' > out.txt" > cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'Rscript /home/korschi/bin/qname_srv.uni.R out.txt outuni.txt' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'samtools view -b -N outuni.txt '${sfolder}'/%/'${fileINN}' > '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'samtools sort '${sfolder}'/%/dd.bam -o '${sfolder}'/%/dd1.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'rm '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		ls -v $sfolder | xargs -I% echo 'mv '${sfolder}'/%/dd1.bam '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
		echo ""; echo "########  umi only"; echo ""
	fi
	if [ $umiB = "2" ]
	then
		# remove duplicates based on samtools markdup (mapping plus barcode)
		ls -v $sfolder | xargs -I% echo 'samtools markdup -r -s -f '${sfolder}'/%/dup.stat.txt -u --barcode-name '${sfolder}'/%/'${fileIN}' '${sfolder}'/%/dd.bam' > cmds/sub1.sh
		echo ""; echo "########  umi samtools markdup"; echo ""
	fi
	if [ $umiB = "3" ]
	then
		# umi_tools (served by pipx),  ? --read-length, maybe away
		ls -v $sfolder | xargs -I% echo 'umi_tools dedup --read-length --umi-separator=: -I '${sfolder}'/%/'${fileIN}' --output-stats='${sfolder}'/%/dedup_stat.txt -S '${sfolder}'/%/dd.bam -L '${sfolder}'/%/dedup.log' > cmds/sub1.sh
		echo ""; echo "########  umi_tools"; echo ""
	fi
	
	ls -v $sfolder | xargs -I% echo 'samtools index '${sfolder}'/%/dd.bam' >> cmds/sub1.sh
	chmod +x cmds/sub1.sh
	cmds/sub1.sh
}

#### STAR - mte
staroutC () {
	pfstarC="03star3out"
	gdi=$gdiSC
	gtf=$gtfSC
	tfa=$tfaSC

	# create result folders
	for sample in `ls $pfin`; do mkdir -p $pfstarC/$sample; done

	## paired end reads  [adjust for dir]
	# transcriptomic coordinates bam (genomic coordinates sam)
	ls -v $pfin | xargs -I% echo 'STAR --runThreadN 8 --genomeDir '$gdi' --sjdbGTFfile '$gtf' --readFilesIn '$pfin'/%/*'$sue'.fastq '$pfin'/%/*'$sue2'.fastq --outFilterMismatchNmax 0 --outFilterMultimapNmax 2 --outSAMmultNmax 1 --outFileNamePrefix '$pfstarC'/%/ --quantMode TranscriptomeSAM' > cmds/mapstC.sh
	chmod +x cmds/mapstC.sh
	cat cmds/mapstC.sh | parallel -P 3 	# 3*8=24 threads = server

	# SAM to BAM when @SQ lines are present in the header
	# *	0	0	0    additional line at the end - for not part of the reference
	ls -v $pfstarC | xargs -I% echo 'samtools sort '${pfstarC}'/%/Aligned.toTranscriptome.out.bam -o '${pfstarC}'/%/Alig.sort.bam' > cmds/sc3C.sh
	ls -v $pfstarC | xargs -I% echo 'samtools index '${pfstarC}'/%/Alig.sort.bam -o '${pfstarC}'/%/Alig.sort.bam.bai' >> cmds/sc3C.sh
	chmod +x cmds/sc3C.sh
	cmds/sc3C.sh
	
	# umi+idx
	barcodeB "Alig.sort.bam" $pfstarC "Aligned.toTranscriptome.out.bam"
	
	# clean up STAR
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/libParams' > cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/logs' >> cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/_STARgenome' >> cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/SJ.out.tab' >> cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Log.progress.out' >> cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Aligned.toTranscriptome.out.bam' >> cmds/cu1c.sh		# comment it out if you want to keep it
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Aligned.out.sam' >> cmds/cu1c.sh
	# other
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/Alig.sort.bam.bai' >> cmds/cu1c.sh
	chmod +x cmds/cu1c.sh
	cmds/cu1c.sh

	# count
	ls -v $pfstarC | xargs -I% echo 'samtools index '${pfstarC}'/%/dd.bam -o '${pfstarC}'/%/dd.bam.bai' > cmds/sc3C.sh
	ls -v $pfstarC | xargs -I% echo 'samtools idxstats '${pfstarC}'/%/dd.bam > '${pfstarC}'/%/mcounts.tsv' >> cmds/sc3C.sh
	chmod +x cmds/sc3C.sh
	cmds/sc3C.sh

	# SALMON
	# alignment-based mode
	ls -v $pfstarC | xargs -I% echo 'salmon quant -p 8 -t '$tfa' -l A -a '${pfstarC}'/%/dd.bam -o '${pfstarC}'/%' > cmds/sal3.sh
	chmod +x cmds/sal3.sh
	cmds/sal3.sh
	
	# clean up samtools & salmon
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/dd.bam.bai' > cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/dd.bam' >> cmds/cu1c.sh
	# salmon
	ls -v $pfstarC | xargs -I% echo 'rm '${pfstarC}'/%/cmd_info.json' >> cmds/cu1c.sh
	ls -v $pfstarC | xargs -I% echo 'rm -r '${pfstarC}'/%/aux_info' >> cmds/cu1c.sh
	chmod +x cmds/cu1c.sh
	cmds/cu1c.sh
}

#### file name list -- (!) ev. needs adjustment
#    works on filelist.txt and creates namelist.txt for R import
fnlA () {
	fl="$(head -n 1 "$tf/filelist.txt")"
	tl=$(tr -dc '/' <<<"$fl" | wc -c)
	numE="$((tl+1))"
	fn=$(cut -d';' -f2 "$tf/filelist.txt" | cut -d'/' -f$numE | cut -d'.' -f1)
	echo $fn
	# save name list
	echo $fn > $tf/namelist.txt
}

#### main pe ####
dt1=$(date)

icreatePE
echo ""; echo "########  initialization -- done"; echo ""
qualconPE
echo ""; echo "########  fastqc -- done"; echo ""
staroutC
echo ""; echo "########  STAR -- mte + salmon + mcounts  done"; echo ""
fnlA
echo ""; echo "########  namelist -- done"; echo ""

echo ""
dt2=$(date)
echo $dt1
echo $dt2


